import { Injectable } from '@angular/core';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { Profile } from '../models';
import { catchError, tap } from 'rxjs/operators';
import { ErrorObservable } from 'rxjs/observable/ErrorObservable';

@Injectable()
export class ProfileService {

    private apiUrl: string = '/bitrix/vuz/api/profile/';
    _current: Observable<Profile> = null;
    _role: string = null;
    current: Profile = new Profile();

    constructor(private http: HttpClient) {
    }

    clearCache() {
        this._current = null;
        this._role = null;
        this.current = null;
    }

    setUrl(url: string) {
        this.apiUrl = url;
    }

    getCurrent(): Observable<Profile> {
        if (!this._current) {
            this._current = this.http.get<any>(this.apiUrl + 'current').pipe(
                tap((profile: Profile) => {
                    this._role = profile.role;
                    this.current = profile;
                }),
                catchError(this.handleError)
            );
        }
        return this._current;
    }

    searchStudents(filter: any): Observable<any> {
        return this.http.post<any>('/bitrix/vuz/api/profiles/students', filter).pipe(
            catchError(this.handleError)
        );
    }

    searchEmployees(filter: any): Observable<any> {
        return this.http.post<any>('/bitrix/vuz/api/profiles/employees', filter).pipe(
            catchError(this.handleError)
        );
    }

    get(id: number): Observable<any> {
        return this.http.get<any>(this.apiUrl + 'view/' + id).pipe(
            catchError(this.handleError)
        );
    }

    getRole(): string {
        if (!this._role) {
            this.getCurrent().subscribe(
                data => {
                    return data.role;
                },
                error => {
                    console.error(error);
                    return null;
                });
        } else {
            return this._role;
        }
    }

    public getType(): Observable<any> {
        return this.http.get<any>(this.apiUrl + 'type').pipe(
            catchError(this.handleError)
        );
    }


    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong,
            console.error(error.error.error.message);
        }
        // return an ErrorObservable with a user-facing error message
        return new ErrorObservable(
            'Something bad happened; please try again later.');
    }
}
