import { Injectable } from '@angular/core';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { catchError } from 'rxjs/operators';
import { ErrorObservable } from 'rxjs/observable/ErrorObservable';

@Injectable()
export class SettingsService {

    private apiUrl: string = '/bitrix/vuz/api/settings/';
    private _valueCache: any = {};
    private _sectionCache: any = {};
    private _cacheDate: any = {};


    constructor(private http: HttpClient) {
    }

    public setUrl(url: string) {
        this.apiUrl = url;
    }

    clearCache(key?: string) {
        if (key) {
            if (this._valueCache.hasOwnProperty(key)) {
                delete this._valueCache[key];
                delete this._cacheDate[key];
            }
            if (this._sectionCache.hasOwnProperty(key)) {
                delete this._sectionCache[key];
                delete this._cacheDate[key];
            }
        } else {
            this._valueCache = {};
            this._sectionCache = {};
            this._cacheDate = {};
        }
    }


    public get(name: string): Observable<any> {
        if (!this._valueCache.hasOwnProperty(name)) {
            const url = this.apiUrl + 'value/' + name;
            this._cacheDate[name] = new Date();

            this._valueCache[name] = this.http.get<any>(url).pipe(
                catchError(this.handleError)
            );
        }
        return this._valueCache[name];
    }

    public getSection(section: string): Observable<any> {
        if (!this._sectionCache.hasOwnProperty(section)) {
            const url = this.apiUrl + 'section/' + section;
            this._cacheDate[section] = new Date();

            this._sectionCache[section] = this.http.get<any>(url).pipe(
                catchError(this.handleError)
            );
        }
        return this._sectionCache[section];
    }

    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong,
            console.error(error.error.error.message);
        }
        // return an ErrorObservable with a user-facing error message
        return new ErrorObservable(
            'Something bad happened; please try again later.');
    }
}
