import { Injectable } from '@angular/core';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
import { User } from '../models';
import { Observable } from 'rxjs/Observable';
import { catchError } from 'rxjs/operators';
import { ErrorObservable } from 'rxjs/observable/ErrorObservable';

@Injectable()
export class UserService {

    private apiUrl: string = '/bitrix/vuz/api/user';

    constructor(private http: HttpClient) {
    }

    setUrl(url: string) {
        this.apiUrl = url;
    }

    getCurrent(): Observable<User> {
        return this.http.get<any>(this.apiUrl + '/').pipe(
            catchError(this.handleError)
        );
    }

    get(id: number): Observable<any> {
        return this.http.get<any>(this.apiUrl + '/' + id).pipe(
            catchError(this.handleError)
        );
    }

    getBx(id: number): Observable<any> {
        return this.http.get<any>(this.apiUrl + '/bx/' + id).pipe(
            catchError(this.handleError)
        );
    }

    search(filter: any): Observable<any> {
        return this.http.post<any>(this.apiUrl + '/search', filter).pipe(
            catchError(this.handleError)
        );
    }

    searchEmployee(filter: any): Observable<any> {
        return this.http.post<any>(this.apiUrl + '/search/employee', filter).pipe(
            catchError(this.handleError)
        );
    }

    private handleError(error: HttpErrorResponse) {
        if (error.error instanceof ErrorEvent) {
            // A client-side or network error occurred. Handle it accordingly.
            console.error('An error occurred:', error.error.message);
        } else {
            // The backend returned an unsuccessful response code.
            // The response body may contain clues as to what went wrong,
            console.error(error.error.error.message);
        }
        // return an ErrorObservable with a user-facing error message
        return new ErrorObservable(
            'Something bad happened; please try again later.');
    }
}
